package com.vp.plugin.sample.captionplacement.actions;

import com.vp.plugin.ApplicationManager;
import com.vp.plugin.DiagramManager;
import com.vp.plugin.action.VPAction;
import com.vp.plugin.action.VPActionController;
import com.vp.plugin.diagram.IComponentDiagramUIModel;
import com.vp.plugin.diagram.IDiagramElement;
import com.vp.plugin.diagram.IDiagramTypeConstants;
import com.vp.plugin.diagram.IDiagramUIModel;
import com.vp.plugin.diagram.IShapeUIModel;
import com.vp.plugin.diagram.shape.IComponentUIModel;
import com.vp.plugin.model.IComponent;
import com.vp.plugin.model.IProject;
import com.vp.plugin.model.factory.IModelElementFactory;

public class CaptionPlacementActionControl implements VPActionController {
	
	// array of possible caption alignment position
	private int[] _alignments = {IShapeUIModel.MODEL_ELEMENT_NAME_ALIGNMENT_ALIGN_TOP_LEFT, 
								 IShapeUIModel.MODEL_ELEMENT_NAME_ALIGNMENT_ALIGN_TOP_MIDDLE,
								 IShapeUIModel.MODEL_ELEMENT_NAME_ALIGNMENT_ALIGN_TOP_RIGHT,
								 IShapeUIModel.MODEL_ELEMENT_NAME_ALIGNMENT_ALIGN_MIDDLE_LEFT,
								 IShapeUIModel.MODEL_ELEMENT_NAME_ALIGNMENT_ALIGN_MIDDLE,
								 IShapeUIModel.MODEL_ELEMENT_NAME_ALIGNMENT_ALIGN_MIDDLE_RIGHT,
								 IShapeUIModel.MODEL_ELEMENT_NAME_ALIGNMENT_ALIGN_BOTTOM_LEFT,
								 IShapeUIModel.MODEL_ELEMENT_NAME_ALIGNMENT_ALIGN_BOTTOM_MIDDLE,
								 IShapeUIModel.MODEL_ELEMENT_NAME_ALIGNMENT_ALIGN_BOTTOM_RIGHT};
	
	@Override
	public void performAction(VPAction arg0) {
		// try to obtain the component diagram from project
		IComponentUIModel componentShape = getComponentFromDiagram();
		if (componentShape != null) {
			// obtain the original name alignment value
			int alignment = componentShape.getModelElementNameAlignment();
			// if alignment is set to follow diagram, then change it as top left
			if (alignment == IShapeUIModel.MODEL_ELEMENT_NAME_ALIGNMENT_FOLLOW_DIAGRAM) {
				alignment = IShapeUIModel.MODEL_ELEMENT_NAME_ALIGNMENT_ALIGN_BOTTOM_RIGHT;
			}
			
			// lookup the index of current alignment value 
			// in _alignments array, and increase index by 1
			for (int i = 0; i < _alignments.length; i++) {
				if (alignment == _alignments[i]) {
					alignment = i+1;
					break;
				}
			}			
			
			// if alignment value is bigger then the length of
			// _alignments array, then use mod to calculate the 
			// reminder value as the new index
			if (alignment >= _alignments.length) {		
				alignment = _alignments.length % alignment;
			}

			// set the new name alignment
			componentShape.setModelElementNameAlignment(_alignments[alignment]);
		}			
	}
	
	private IComponentUIModel getComponentFromDiagram() {
		// obtain the project and loop through the diagrams
		IProject project = ApplicationManager.instance().getProjectManager().getProject();
		
		// loop through the diagrams to locate
		// the one call "Caption Placement Test"
		IDiagramUIModel[] diagrams = project.toDiagramArray();		
		if (diagrams != null) {
			for (IDiagramUIModel diagram : diagrams) {
				if (diagram instanceof IComponentDiagramUIModel && "Caption Placement Test".equals(diagram.getName())) {
					// once the diagram was found, then see does 
					// it contain a component call "MyComponent"
					IDiagramElement[] elements = diagram.toDiagramElementArray();
					if (elements != null) {
						for (IDiagramElement element : elements) {
							// return the diagram if "MyComponent" was found
							if (element instanceof IComponentUIModel && "MyComponent".equals(element.getModelElement().getName())) {
								return (IComponentUIModel) element;								
							}
						}						
					}										
				}
			}			
		}
		return createComponent();
	}
	
	public IComponentUIModel createComponent() {
		DiagramManager diagramManager = ApplicationManager.instance().getDiagramManager();
		
		// create component diagram
		IComponentDiagramUIModel diagram = (IComponentDiagramUIModel) diagramManager.createDiagram(IDiagramTypeConstants.DIAGRAM_TYPE_COMPONENT_DIAGRAM);
		diagram.setName("Caption Placement Test");
		
		// create component shape
		IComponent componentModel = IModelElementFactory.instance().createComponent();
		componentModel.setName("MyComponent");
		IComponentUIModel componentShape = (IComponentUIModel) diagramManager.createDiagramElement(diagram, componentModel);
		componentShape.setBounds(100, 100, 500, 400);
		componentShape.setRequestResetCaption(true);
		
		// show up the diagram		
		diagramManager.openDiagram(diagram);
		
		// return the component shape
		return componentShape;
	}

	@Override
	public void update(VPAction arg0) {
		// TODO Auto-generated method stub
		
	}

}
